/**
    Battery analyzer
    (c) 2016-2017 F. Štefanec
    This project is provided WITHOUT ANY WARRANTY, use at your own risk.
**/


#ifndef MAIN_H_INCLUDED
#define MAIN_H_INCLUDED

/** Library inclusions **/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>
#define uint_farptr_t unsigned long int // to fix some undefined stuff
#include <avr/pgmspace.h>

/** Pin/port definitions **/

#define BTN_PIN PINB
#define BTN_PORT PORTB

#define BTN_ESC_PIN PB4
#define BTN_UP_PIN PB2
#define BTN_DOWN_PIN PB1
#define BTN_OK_PIN PB0

#define INHIBIT_PIN PC2
#define INHIBIT_PORT PORTC
#define INHIBIT_DDR DDRC

#define BUZZER_PIN PB5
#define BUZZER_PORT PORTB
#define BUZZER_DDR DDRB

#define MUX_BATT_VOLTAGE (0)
#define MUX_SUPPLY_VOLTAGE (1<<MUX0)
#define MUX_TEMPERATURE (1<<MUX1)

#define FAN_PIN PC4
#define FAN_PORT PORTC
#define FAN_DDR DDRC

/** Beep lengths **/

#define BEEP_LEN_BUTTON 10
#define BEEP_LEN_FINISHED 250
#define BEEP_LEN_REPEAT 5
#define BEEP_LEN_START 5

/** Custom settings **/

#define AD_AVERAGE_COUNT 8

uint8_t x, y, z; /// general variables


/** Status and menu variables **/

uint8_t stat, stat2, settings; // status variable
uint8_t submenu_stat;
uint8_t field;
uint8_t menu_pos = 1, old_menu_pos = 1, menu_pos2, old_menu_pos2;
volatile uint8_t beep_counter, old_beep_counter;
uint8_t rin_current_choice = 0;
const uint8_t rin_current1[10] PROGMEM = {0,  25,   0,  50,   0, 100, 150,   0,  50,   0};
const uint8_t rin_current2[10] PROGMEM = {25, 50,  50, 100, 100, 200, 250, 200, 250, 250};

/** Timing related variables **/

volatile uint8_t seconds;
volatile uint8_t minutes;
volatile uint8_t hours;
unsigned long long elapsed_secs = 0; // max 917999 seconds
uint8_t tim0_sw_pre; // software prescaler for Timer 0

/** Button handling variables **/

uint8_t buttons_old_pin;
uint8_t buttons;
uint8_t buttons_old;
uint8_t buttons_pressed;
uint8_t button_repeat_counter;
uint8_t btn_debounce = 0; // button debounce counter

/** Display variables **/

uint8_t batt_icon = 0; // discharging battery icon
char buf[20]; // LCD buffer
char disch_power_str[8];

/** Measurement variables **/

uint16_t term_voltage = 80; // termination voltage in 10mV
volatile uint16_t adc_voltage = 0; // voltage in 10mV, max 20.46V (>20V = error)
uint16_t disp_voltage = 0;
volatile uint16_t supp_voltage = 0; // AD value for supply voltage
volatile uint16_t temperature_value = 0; // AD value for temperature
uint16_t volt_buf[64];
uint8_t ad_buf_ptr = 0;
uint16_t ad_avg_buf[8];
uint8_t ad_avg_buf_ptr;
uint8_t ad_throwaway;
uint8_t pwm_current = 1; // current in 10mA, max 2.55A
uint16_t v1, v2, i1, i2=25, rin; // (Rin measurement)

unsigned long long mAs = 0; // milliampseconds (max 917999*2550 = 2340897450 mAs)
unsigned long long mAh100 = 0; // milliamphours*100 (max 2340897450/36 = 65024929 mAh100 - to give 0.01mAh accuracy)

int8_t offset_comp = +105; // offset compensation, in 0,000625 volts, this will be added to the displayed voltage

// MAX

// 2.55A 20.46V 255 2046


/** Function prototypes **/

void terminate_discharge();
void calculate_mAh();
void mAh_string(char *buf);
void time_string(char *buf);
void voltage_string(char *buf, uint16_t voltage);
void current_string(char *buf, uint8_t current);
void watt_string(char* buf, uint16_t voltage, uint8_t current);
void display_prog();
void display_ccload();
void terminate_discharge();
void enable_1s_timer();
void fast_1s_timer();
void disable_1s_timer();
void set_current(uint8_t current);
void rin_test();

void lcd_print_pmem(const char *data, uint8_t length);
void lcd_print_pmem_setpos(const char *data, uint8_t len, uint8_t column, uint8_t line);
uint8_t get_buttons();
void clear_time();
void dly_1sec();

void enable_wdt();
void disable_wdt();


void halt_error();

void cap_submenu();
void cc_submenu();
void rin_submenu();
void settings_submenu();
void escape_to_menu();
void print_menu(const char **items, uint8_t position);
void service_menu();
void disp_menu();
void settings_submenu();
void print_settings_submenu(uint8_t cursor_pos);
void calib_menu();

void save_eeprom();
void set_offset_comp();
void save_eeprom();
void load_eeprom();

#endif
